from ...domain.models.metric_type import MetricType, MetricTypeBase
from ...utils.string import build_query_params
from ...infrastructure.clients.http_client import Client
from typing import List

class MetricTypeService:

    def __init__(self, client: Client):
        self._client = client

    def create(self, name: str, criteria: str | None = None, evaluation_steps: list[str] | None = None):
        """
        Create a new metric type.
        
        Args:
            name (str): Name of the metric type.
            criteria (str, optional): Criteria for the metric type.
            evaluation_steps (list[str], optional): Evaluation steps for the metric type.
            
        Returns:
            MetricType: The created metric type object.
        """
        try:
            metric_type = MetricTypeBase(
                name=name,
                criteria=criteria,
                evaluation_steps=evaluation_steps,
            )

            metric_type.model_validate(metric_type.model_dump())
            response = self._client.post(f"metricTypes", json=metric_type.model_dump(by_alias=True))
            metric_type_response = MetricType(**response.json())
            
            return metric_type_response
        except Exception as e:
            print(f"Error creating Metric Type: {e}")
            return None

    def get(self, metric_type_id: str):
        """
        Retrieve a metric type by its ID.
        
        Args:
            metric_type_id (str): ID of the metric type to retrieve.
            
        Returns:
            MetricType: The retrieved metric type object.
        """
        response = self._client.get(f"metricTypes/{metric_type_id}")
        return MetricType(**response.json())

    def delete(self, metric_type_id: str):
        """
        Delete a metric type by its ID.
        
        Args:
            metric_type_id (str): ID of the metric type to delete.
            
        Returns:
            MetricType: Deleted metric type object.
        """
        self._client.delete(f"metricTypes/{metric_type_id}")
    
    def list(self, offset: int = None, limit: int = None) -> List[MetricType]:
        """
        Get a list of metric types.
        
        Args:
            offset (int, optional): Offset for pagination.
            limit (int, optional): Limit for pagination.
            
        Returns:
            list[MetricType]: List of metric types.
        """
        query_params = build_query_params(offset=offset, limit=limit)
        response = self._client.get(f"metricTypes?{query_params}")
        metric_types = [MetricType(**metric_type) for metric_type in response.json()]
        return metric_types



from ...domain.models.version import VersionBase, Version, VersionBaseOptionalProps
from ...utils.string import sanitize_string, build_query_params
from ...infrastructure.clients.http_client import Client

class VersionService:
    def __init__(self, client: Client):
        self._client = client

    def version_exists(self, version_name: str) -> bool:
        """
        Check if a version exists by its name.
        
        Args:
            version_name (str): Name of the version.
            
        Returns:
            bool: True if the version exists, False otherwise.
        """
        version = self._client.versions(version_name)
        return bool(version)

    def create(self, product_id: str, name: str, optional_props: dict[str, str] = {}):
        """
        Given a version name, create a new version if it doesn't exist.
        
        Args:
            product_id (str): ID of the product.
            name (str): Name of the version.
            optional_props (dict[str, str], optional): Optional properties for the version.
            
        Returns:
            Version: The created version object.
        """
        for key, _ in optional_props.items():
            if key not in VersionBaseOptionalProps.__fields__:
                raise ValueError(f"Invalid key: {key}. Must be one of: {', '.join(VersionBaseOptionalProps.__fields__.keys())}")
        
        try:
            name = sanitize_string(name)
            version = VersionBase(
                name=name,
                product_id=product_id,
                **optional_props
            )
            version.model_validate(version.model_dump())
            response = self._client.post(f"versions", json=version.model_dump(by_alias=True))
            version_response = Version(**response.json())
            return version_response
        except Exception as e:
            print(f"Error creating version {name}: {e}")
            return None

    def get(self, version_id: str):
        """
        Retrieve a version by its ID.
        
        Args:
            version_id (str): ID of the version to retrieve.
            
        Returns:
            Version: The retrieved version object.
        """
        response = self._client.get(f"versions/{version_id}")
        return Version(**response.json())

    def list(self, product_id: str, offset: int = None, limit: int = None):
        """
        Get a list of versions for a given product.
        
        Args:
            product_id (str): ID of the product.
            offset (int, optional): Offset for pagination.
            limit (int, optional): Limit for pagination.
            
        Returns:
            list[Version]: List of version objects.
        """
        query_params = build_query_params(productIds=[product_id], offset=offset, limit=limit)
        response = self._client.get(f"versions?{query_params}")
        versions = [Version(**version) for version in response.json()]
        return versions

    def delete(self, version_id: str):
        """
        Delete a version by its ID.
        
        Args:
            version_id (str): ID of the version to delete.
            
        Returns:
            Version: Deleted version object.
        """
        self._client.delete(f"versions/{version_id}")


from datetime import datetime
from enum import Enum

from ...utils.from_camel_case_base_model import FromCamelCaseBaseModel

class TestBase(FromCamelCaseBaseModel):
    name: str
    type: str
    product_id: str
    ground_truth_uri: str | None = None
    uri: str | None = None

class Test(TestBase):
    id: str
    created_at: datetime | None = None
    deleted_at: datetime | None = None


from ...utils.from_camel_case_base_model import FromCamelCaseBaseModel

class EvaluationTaskBase(FromCamelCaseBaseModel):
  evaluation_id: str
  input: str
  actual_output: str
  expected_output: str | None
  context: str | None

class EvaluationTask(EvaluationTaskBase):
  id: str
  metric_type_id: str
  user_id: str | None = None
  status: str
  score: float | None = None
  reason: str | None = None
  error: str | None = None
  created_at: str | None
  deleted_at: str | None = None
  evaluated_at: str | None = None


from ...utils.from_camel_case_base_model import FromCamelCaseBaseModel

class MetricTypeBase(FromCamelCaseBaseModel):
    name: str
    criteria: str | None = None
    evaluation_steps: list[str] | None = None

class MetricType(MetricTypeBase):
    id: str
    organization_id: str
    created_at: str
    deleted_at: str | None = None


from ...utils.from_camel_case_base_model import FromCamelCaseBaseModel

class VersionBaseOptionalProps(FromCamelCaseBaseModel):
    dataset_description: str | None = None
    dataset_uri: str | None = None
    description: str | None = None
    endpoint: str | None = None
    foundational_model: str | None = None
    guardrails: str | None = None
    provider: str | None = None
    system_prompt: str | None = None

class VersionBase(VersionBaseOptionalProps):
    name: str
    product_id: str

class Version(VersionBase):
    id: str
    created_at: str
    deleted_at: str | None = None


from ...infrastructure.clients.http_client import Client
from galtea.domain.models.evaluation_task import EvaluationTask, EvaluationTaskBase
from galtea.utils.string import build_query_params

class EvaluationTaskService:
    def __init__(self, client: Client):
        self._client = client

    def create(self, metrics: list[str], evaluation_id: str, input: str, actual_output: str, expected_output: str = None, context: str = None):
        """
        Given an evaluation id, create a new evaluation task to evaluate the parameter given for the metrics.
        
        Args:
            metrics (list[str]): List of metric type names.
            evaluation_id (str): ID of the evaluation.
            input (str): Input for the evaluation task.
            actual_output (str): Actual output for the evaluation task.
            expected_output (str, optional): Expected output for the evaluation task.
            context (str, optional): Context for the evaluation task.

        Returns:
            list[EvaluationTask]: List of evaluation tasks created.
        """
        try:
            # Create the base object with the fields it supports
            evaluation_task = EvaluationTaskBase(
                evaluation_id=evaluation_id,
                input=input,
                actual_output=actual_output,
                expected_output=expected_output,
                context=context,
            )
            
            # Validate the base object
            evaluation_task.model_validate(evaluation_task.model_dump())
            
            # Create a dictionary with all fields including metric_type_names
            request_body = evaluation_task.model_dump(by_alias=True)
            request_body["metricTypeNames"] = metrics
            
            # Send the request with the complete body
            response = self._client.post(f"evaluationTasks", json=request_body)
            evaluation_tasks = [EvaluationTask(**evaluation_task) for evaluation_task in response.json()]
            return evaluation_tasks
        except Exception as e:
            print(f"Error creating evaluation task: {e}")
            return None

    def get(self, evaluation_task_id: str):
        """
        Retrieve an evaluation task by its ID.
        
        Args:
            evaluation_task_id (str): ID of the evaluation task to retrieve.
            
        Returns:
            EvaluationTask: The retrieved evaluation task object.
        """
        response = self._client.get(f"evaluationTasks/{evaluation_task_id}")
        return EvaluationTask(**response.json())

    def list(self, evaluation_id: str = [], offset: int = None, limit: int = None):
        """
        Get a list of evaluation tasks for a given evaluation.
        
        Args:
            evaluation_id (str): ID of the evaluation.
            offset (int, optional): Offset for pagination.
            limit (int, optional): Limit for pagination.
            
        Returns:
            list: List of evaluation tasks.
        """
        query_params = build_query_params(evaluationIds=[evaluation_id], offset=offset, limit=limit)
        response = self._client.get(f"evaluationTasks?{query_params}")
        evaluation_tasks = [EvaluationTask(**evaluation_task) for evaluation_task in response.json()]
        return evaluation_tasks

    def delete(self, evaluation_task_id: str):
        """
        Delete an evaluation task by its ID.
        
        Args:
            evaluation_task_id (str): ID of the evaluation task to delete.
            
        Returns:
            EvaluationTask: Deleted evaluation task object.
        """
        self._client.delete(f"evaluationTasks/{evaluation_task_id}")


from ...domain.models.test import TestBase, Test
from ...utils.string import build_query_params
from ...infrastructure.clients.http_client import Client
from ...utils.s3 import upload_file_to_s3, download_file_from_s3
import os

class TestService:
    def __init__(self, client: Client):
        self._client = client

    def __generate_and_upload_presigned_url(self, file_path: str):
        presigned_url_response = self._client.get("s3/generate-put-presigned-url", {
            "key": os.path.basename(file_path)
        })

        if "uploadPresignedUrl" in presigned_url_response.json():
            presigned_url = presigned_url_response.json().get("uploadPresignedUrl")
        else:
            raise Exception(f"Failed to generate presigned URL")
        
        if not upload_file_to_s3(file_path, presigned_url):
            raise Exception(f"Failed to upload file to {presigned_url}")

        return presigned_url

    def create(self, name: str, type: str, product_id: str, ground_truth_file_path: str | None = None, test_file_path: str | None = None):
        """
        Given a test name, create a new test if it doesn't exist.
        
        Args:
            name (str): Name of the test.
            type (str): Type of the test.
            product_id (str): Product ID of the test.
            ground_truth_file_path (str, optional): Path to the ground truth file to be uploaded.
            test_file_path (str, optional): Path to the test file to be uploaded.
            
        Returns:
            Test: The created test object.
        """
        ground_truth_url = self.__generate_and_upload_presigned_url(ground_truth_file_path) if ground_truth_file_path else None
        test_presigned_url = self.__generate_and_upload_presigned_url(test_file_path) if test_file_path else None

        test = TestBase(
            name=name,
            type=type,
            product_id=product_id,
            ground_truth_uri=ground_truth_url,
            uri=test_presigned_url
        )
        test.model_validate(test.model_dump())

        response = self._client.post("tests", json=test.model_dump(by_alias=True))
        test_response = Test(**response.json())

        return test_response

    def get(self, test_id: str):
        """
        Retrieve a test by its ID.
        
        Args:
            test_id (str): ID of the test to retrieve.
            
        Returns:
            Test: The retrieved test object.
        """
        response = self._client.get(f"tests/{test_id}")
        return Test(**response.json())

    def download(self, test: Test, output_directory: str):
        """
        Download a test file from S3 using the presigned URL.
        
        Args:
            test (Test): Test object.
            output_directory (str): Directory where the file will be downloaded.
            
        Returns:
            str: Path to the downloaded file.
        """
        query_params = build_query_params(s3Uri=test.uri)
        response = self._client.get(f"s3/generate-get-presigned-url?{query_params}")
        if "downloadPresignedUrl" in response.json():
            download_url = response.json().get("downloadPresignedUrl")
        else:
            raise Exception(f"Failed to generate download URL")
        
        return download_file_from_s3(download_url, os.path.basename(test.uri), output_directory)

    def list(self, product_id: str, offset: int = None, limit: int = None):
        """
        List all tests for a given product.
        
        Args:
            product_id (str): ID of the product.
            offset (int, optional): Offset for pagination.
            limit (int, optional): Limit for pagination.
            
        Returns:
            list[Test]: List of test objects.
        """
        query_params = build_query_params(productIds=[product_id], offset=offset, limit=limit)
        response = self._client.get(f"tests?{query_params}")
        tests = [Test(**test) for test in response.json()]
        return tests

    def delete(self, test_id: str):
        """
        Delete a test by its ID.
        
        Args:
            test_id (str): ID of the test to delete.
            
        Returns:
            Test: Deleted test object.
        """
        self._client.delete(f"tests/{test_id}")


from galtea.application.services.evaluation_task_service import EvaluationTaskService
from galtea.application.services.metric_type_service import MetricTypeService
from .application.services.product_service import ProductService
from .application.services.test_service import TestService
from .application.services.version_service import VersionService
from .application.services.evaluation_service import EvaluationService
from .infrastructure.clients.http_client import Client
from termcolor import colored

class Galtea:
  def __init__(self, api_key: str):
    self.__client = Client(api_key)
    self.products = ProductService(self.__client)
    self.tests = TestService(self.__client)
    self.versions = VersionService(self.__client)
    self.metrics = MetricTypeService(self.__client)
    self.evaluations = EvaluationService(self.__client)
    self.evaluation_tasks = EvaluationTaskService(self.__client)
  
  def evaluate(self, metrics: list[str] , evaluation_id: str, input: str, actual_output: str, expected_output: str = None, context: str = None):
    """
    Given an evaluation id, create a set of evaluation tasks to evaluate your product based on the metrics provided.
    This function will create a new evaluation task for each metric type provided in the list.
    It will also print the details of each evaluation task created.
    If no evaluation task was created, it will print an error message.
    
    Args:
      metrics (list[str]): List of metric type names.
      evaluation_id (str): ID of the evaluation.
      input (str): Input for the evaluation task.
      actual_output (str): Actual output for the evaluation task.
      expected_output (str, optional): Expected output for the evaluation task.
      context (str, optional): Context for the evaluation task.
      
    Returns:
      list[EvaluationTask]: List of evaluation tasks created.
    """
    evaluation_tasks = self.evaluation_tasks.create(metrics=metrics, evaluation_id=evaluation_id, input=input, actual_output=actual_output, expected_output=expected_output, context=context)
    
    if evaluation_tasks is None or len(evaluation_tasks) == 0:
      print(colored("No evaluation task was created", 'red'))
      return None

    for evaluation_task in evaluation_tasks:      
      if evaluation_task is None:
          print(colored("Error creating evaluation task", 'red'))
          return None
      print(colored("----------------------------------------------------------------", 'green'))
      print(colored(f"→ evaluation_task id: {evaluation_task.id}", 'green'))
      print(colored(f"→ evaluation id: {evaluation_task.evaluation_id}", 'green'))
      print(colored(f"→ metric_id: {evaluation_task.metric_type_id}", 'green'))
      print(colored(f"→ input: {evaluation_task.input}", 'green'))
      print(colored(f"→ actual_output: {evaluation_task.actual_output}", 'green'))
      print(colored(f"→ expected_output: {evaluation_task.expected_output}", 'green'))
      print(colored(f"→ context: {evaluation_task.context}", 'green'))
      print(colored("----------------------------------------------------------------", 'green'))

    return evaluation_tasks
  
